using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace Reveal
{
    public partial class Form1 : Form
    {
        Rectangle m_rectView = new Rectangle(0, 0, 200, 200);
        GraphicsPath m_pathClip = new GraphicsPath();
        MyPoint m_ptStart = null;
        Region m_rgnClip = null;
        BufferedGraphicsContext bufferContext;

        public Form1()
        {
            InitializeComponent();
            m_pathClip.AddEllipse(m_rectView);
            m_rgnClip = new Region(m_pathClip);
            bufferContext = new BufferedGraphicsContext();
            bufferContext.MaximumBuffer = panel1.Size;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            m_pathClip.Dispose();
            m_rgnClip.Dispose();
            bufferContext.Dispose();
            base.OnClosing(e);
        }

        private void PaintHelper(Graphics g)
        {
            if (clipToolStripMenuItem.Checked)
            {
                g.Clip = m_rgnClip;
            }
            using (Pen pen = new Pen(Color.Red, 1))
            {
                pen.Alignment = PenAlignment.Inset;
                g.DrawEllipse(Pens.Red, m_rectView);
            }
            using (StringFormat format = new StringFormat())
            {
                format.Alignment = StringAlignment.Center;
                format.LineAlignment = StringAlignment.Center;
                for (int i = 0; i < 10; i++)
                {
                    for (int j = 0; j < 10; j++)
                    {
                        Rectangle rect = new Rectangle(j * 55, i * 55, 50, 50);
                        g.DrawRectangle(Pens.Black, rect);
                        int number = j + i * 10;
                        g.DrawString(number.ToString(), this.Font, Brushes.Black, rect, format);
                    }
                }
            }
            g.DrawEllipse(Pens.Red, m_rectView);
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {            
            PaintHelper(e.Graphics);
        }

        private void panel1_MouseDown(object sender, MouseEventArgs e)
        {
            MyPoint ptMouse = new MyPoint(e.X, e.Y);
            if (m_rectView.Contains(ptMouse))
            {
                m_ptStart = ptMouse;
            }
        }

        void RenderGraphics()
        {
            using (Graphics g = panel1.CreateGraphics())
            {
                using (BufferedGraphics frame =
                    bufferContext.Allocate(g, panel1.ClientRectangle))
                {
                    frame.Graphics.FillRectangle(Brushes.Wheat, panel1.ClientRectangle);
                    PaintHelper(frame.Graphics);
                    frame.Render();
                }
            }
        }

        private Rectangle InflateCopy(Rectangle rect, Size sz)
        {
            Rectangle rectInflate = new Rectangle(rect.Location, rect.Size);
            rectInflate.Inflate(sz);
            return rectInflate;
        }

        private void panel1_MouseMove(object sender, MouseEventArgs e)
        {
            Control control = (Control)sender;
            if (m_ptStart != null)
            {
                using (GraphicsPath path = new GraphicsPath())
                {
                    if (control is Form || !panelToolStripMenuItem1.Checked)
                    {
                        control.Invalidate(InflateCopy(m_rectView, new Size(2,2)));
                    }

                    MyPoint ptMouse = new MyPoint(e.X, e.Y);
                    MyPoint ptOffset = ptMouse - m_ptStart;
                    m_ptStart = ptMouse;
                    m_rectView = m_rectView + ptOffset;
                    path.AddEllipse(m_rectView);
                    m_rgnClip = new Region(path);

                    if (control is Panel && panelToolStripMenuItem1.Checked)
                    {
                        RenderGraphics();
                    }
                    else
                    {
                        control.Invalidate(InflateCopy(m_rectView, new Size(2, 2)));
                    }
                }
            }
        }

        private void panel1_MouseUp(object sender, MouseEventArgs e)
        {
            m_ptStart = null;
        }

        private void buttonBuffer_Click(object sender, EventArgs e)
        {
            
        }

        private void Form1_MouseDown(object sender, MouseEventArgs e)
        {
            panel1_MouseDown(sender, e);
        }

        private void Form1_MouseMove(object sender, MouseEventArgs e)
        {
            panel1_MouseMove(sender, e);
        }

        private void Form1_MouseUp(object sender, MouseEventArgs e)
        {
            panel1_MouseUp(sender, e);
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            PaintHelper(e.Graphics);
        }

        private void panelToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            Invalidate(true);
        }

        private void hideToolStripMenuItem_Click(object sender, EventArgs e)
        {
            panel1.Visible = !panel1.Visible;
            Invalidate(true);
        }

        private void panelToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void buttonBuffer_Click_1(object sender, EventArgs e)
        {
         
        }

        private void doubleBufferToolStripMenuItem_Click(object sender, EventArgs e)
        {
            
        }

        private void mainToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer, 
                          mainToolStripMenuItem.Checked);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint,
                          mainToolStripMenuItem.Checked);
            Invalidate(true);
        }

        private void panelToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            Invalidate(true);
            
        }

        private void doubleBufferToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            mainToolStripMenuItem.Enabled = !panel1.Visible;
            panelToolStripMenuItem1.Enabled = panel1.Visible
                && !autoPanelToolStripMenuItem.Checked;
            autoPanelToolStripMenuItem.Enabled = panel1.Visible
                && !panelToolStripMenuItem1.Checked;
            Invalidate(true);
        }

        private void clipToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Invalidate(true);
        }

        private void autoPanelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            panel1.SetBuffered(autoPanelToolStripMenuItem.Checked);
            Invalidate(true);
        }
    }
}